declare class CountDownTimer {
	constructor(millisInFuture: number, countDownInterval: number)

	/**
	 * 开启倒计时
	 */
	start(): void;

	/**
	 * 取消倒计时
	 */
	cancel(): void;

	/**
	 * 添加倒计时监听器
	 * @param listener
	 */
	addCountDownListener(listener: CountDownCallback): void;

	/**
	 * 添加结束监听器
	 * @param listener
	 */
	addFinishListener(listener: FinishCallback): void;
}

type CountDownCallback = (time: TimeInfo) => void;
type FinishCallback = () => void;

declare class TimeInfo {
	time: number;
	day: number;
	hour: number;
	minute: number;
	second: number;

	format(value: number): number
}

declare interface CalendarHelper {
	/**
	 * 获取当前日期字符串 YYYY-MM-DD     2019-06-01
	 */
	getCurrentFormatDateString(): string;

	/**
	 * 获取特定月份的日历数据
	 * @param year
	 * @param month
	 */
	getCalendarData(year: number, month: number): MonthInfo;

	/**
	 * 获取上个月的数据
	 * @param year
	 * @param month
	 */
	getLastMonthData(year: number, month: number): MonthInfo;

	/**
	 * 获取下个月的日历数据
	 * @param year
	 * @param month
	 */
	getNextMonthData(year: number, month: number): MonthInfo;

	/**
	 * 获取当前月份的日历数据
	 */
	getCurrentMonthData(): MonthInfo;

	/**
	 * 获取日历基础数据(从当前月，往后推)
	 * @param count
	 * @returns {Array}
	 */
	getCalendarDataList(count): Array<MonthInfo>;
}

declare class DayInfo {
	year: number;
	month: number;
	day: number;
	isSupplementDay: boolean;
	titleName: string;
	formatDay: string;
	time: number;

	constructor(year, month, day);

	setSupplementDay(isSupplement: Boolean): void;

	getFormatDayString(): string;

	equal(object: DayInfo): boolean;

	toString(): string;
}

declare class Month {
	year: number;
	month: number;
}

declare class MonthInfo extends Month {
	titleName: string;
	year: number;
	month: number;

	constructor(year, month);

	getDayItems(): Array<DayInfo>;

	toString(): string;
}

declare interface ObjectHelper {
	/**
	 * 克隆对象
	 * @param obj 需要克隆的原始对象
	 */
	cloneObject(obj: object): object;

	/**
	 * 获取继承对象。类似java的继承实现
	 * @param targetObject 子对象
	 * @param sourceObject  父对象
	 */
	extendObject(targetObject: object, sourceObject: object): object;

	/**
	 * 判断列表是否包含特定的对象
	 * @param list  对象列表
	 * @param object  对象
	 */
	containObject(list: Array<object>, object: object): boolean;

	/**
	 * 深度比较对象是否相等
	 * @param obj1
	 * @param obj2
	 */
	deepEqual(obj1: object, obj2: object): boolean;

	/**
	 * 从对象中获取值
	 * @param obj {a:{b:{c:{text:"10}}}}
	 * @param keyStr  "a.b.c.text"
	 */
	getValueFromObject(obj: object, keyStr: string): any;

	/**
	 * 根据字段列表获取对象
	 * @param keysList 字段列表
	 * @param valuesObject 包含当前数据的原始对象
	 */
	getObjectByList(keysList: Array<string>, valuesObject: object): object;

	/**
	 *获取目标对象
	 * @param keysObject 需要获取的目标对象
	 * @param targetObject 原始目标对象
	 */
	getObjectByObject(keysObject: object, targetObject: object): object;

	/**
	 * 设置监听器
	 * @param data 监听数据源
	 * @param watch  监听对象
	 */
	setWatcher(data: object, watch: object): void;
}

type DateParams = string | number | Date;

declare interface DateHelper {
    /**
     * 获取日期对象（兼容ios下 yyyy-MM-ddd 格式日期转换异常的问题 ）
      * @param date
     */
	getDateInstance(date: DateParams): Date;

	/**
	 * 判断是不是今天
	 * @param date 日期
	 */
	isToday(date: DateParams): boolean;

	/**
	 * 格式化日期对象
	 * @param date  Date对象或者 创建Date对象的构造参数
	 * @param formatStyle 格式化样式字符串 如：yyyy-MM ; YYYY-MM-DD  HH:mm:ss ; yyyy-MM-dd  hh:mm:SS
	 */
	formatDateString(date: DateParams, formatStyle: string): string;

	/**
	 * 获取是星期几
	 * @param date
	 * @param isEnglish
	 */
	getWeekName(date: DateParams, isEnglish?: boolean): string;

	/**
	 * 获取月份天数
	 * @param date  Date 对象、时间戳
	 */
	getDaysInMonth(date: DateParams): number;

	/**
	 * 判断是否是闰年
	 * @param year
	 */
	isLeapYear(year: number): boolean;

	/**
	 *  获取指定月份天数
	 * @param year 年
	 * @param month 月
	 */
	getMonthDays(year: number, month: number): number;

	/**
	 * 格式化数字
	 * @param num
	 */
	formatNumber(num: number): string

	/**
	 * 获取当前日期 YYYY-MM-DD
	 */
	getCurrentFormatDate(): string;

	/**
	 * 获取上个月的年月信息
	 * @param year
	 * @param month
	 */
	getLastMonthInfo(year: number, month: number): Month;

	/**
	 * 获取下个月的年月信息
	 * @param year
	 * @param month
	 */
	getNextMonthInfo(year: number, month: number): Month;

	/**
	 * 计算两日期相差多少年
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffYears(startTime: number, endTime: number): number;

	/**
	 * 计算两日期相差多少年
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffMonths(startTime: number, endTime: number): number;

	/**
	 * 计算两日期相差多少天
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffDays(startTime: number, endTime: number): number;

	/**
	 * 计算两日期相差多少小时
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffHours(startTime: number, endTime: number): number;

	/**
	 * 计算两日期相差多少分
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffMinutes(startTime: number, endTime: number): number;

	/**
	 * 计算两日期相差多少秒
	 * @param startTime 起始时间毫秒数
	 * @param endTime 结束时间毫秒数
	 */
	diffSeconds(startTime: number, endTime: number): number;
}

declare interface UrlHelper {
	/**
	 * 获取(携带查询参数)地址
	 * @param baseUrl 基本地址
	 * @param params  参数对象
	 */
	getQueryUrl(baseUrl: string, params: Object): string;

	/**
	 * 从地址上获取参数集合
	 * @param url  地址
	 */
	getQueryMapFromUrl(url: string): Map<string, any>;
}

/**
 * 日期常量
 */
declare interface DateConstants {
	/**
	 * 一天的毫秒数
	 */
	dayMillis: number;
	/**
	 * 一个小时的毫秒数
	 */
	hourMillis: number;
	/**
	 * 一分钟的毫秒数
	 */
	minuteMillis: number;
	/**
	 * 一秒种的毫秒数
	 */
	secondMillis: number;
}

declare interface ArrayHelper {
	/**
	 * 将数据分组
	 * @param list 原始数据列表
	 * @param groupNumber 分组的个数
	 */
	groupingList(list: Array<any>, groupNumber: number)
	/**
	 * 补齐列表数据，使其是分组数的整数倍
	 * 返回新的数据
	 * @param list
	 * @param groupNumber
	 * @returns {*}
	 */
	makeUpGroupList(list: Array<any>, groupNumber: number)
}

declare interface StringHelper {

	/**
	 * 判读字符串是不是空值
	 * @param str
	 */
	isEmpty(str): string;

	/**
	 * 校验手机号（严格校验）
	 * //移动号段：134 135 136 137 138 139 147 148 150 151 152 157 158 159  165 172 178 182 183 184 187 188 195 198
	 * // 联通号段：130 131 132 145 146 155 156 166 167 171 175 176 185 186
	 * // 电信号段：133 149 153 173 174 177 180 181 189  191  199
	 * // 虚拟运营商:170
	 * 分组如下
	 * //130 131 132 133 134 135 136 137 138 139
	 * //145 146 147 148 149
	 * //150 151 152 153 155 156 157 158 159
	 * // 165 166 167
	 * //170  171  172 173 174  175 176 177 178
	 * //180 181 182 183 184 185 186 187 188 189
	 * // 191  195  198 199
	 * @param phone
	 * @returns {boolean}
	 */
	isPhoneNumber(phone: string | number): boolean;

	/**
	 * 判断是不是身份证
	 * @param codeStr
	 */
	isIDCard(codeStr: string): boolean;

	/**
	 * 判断是不是邮箱
	 * @param email
	 */
	isEmail(email: string): boolean;

	/**
	 * 从富文本中，查找img 标签，收集图片地址(只支持.png .jpg .jpeg 结尾)
	 * @param richText 富文本
	 * @returns {[]}  返回图片列表
	 */
	getImageListFromRichText(richText: string): Array<string>

	/**
	 * 将整数部分逢三一断
	 */
	formatNumber(value: number | string): string;

	/**
	 * *RGB颜色转换为16进制
	 * @param rgbColor RGB颜色
	 */
	colorRbgToHex(rgbColor: string): string;

	/**
	 *16进制颜色转为RGB格式
	 * @param hexColor  16进制颜色
	 * @returns {string}  RGB格式颜色
	 */
	colorHexToRgb(hexColor: string): string;

	/**
	 * 替换所有
	 * @param content 当前文本
	 * @param pattern 旧字符串或者正则表达式
	 * @param newStr 要替换的内容
	 * @returns {*}  返回新的字符串
	 */
	replaceAll(content: string, pattern: RegExp | string, newStr: string): string;
}

declare class WatcherHelper {
	setWatcher(data: object, watch: object): void
}

type SubscriberCallback = (msg: any) => void;

declare interface EventBus {
	postSticky(msg: any, eventType: string): void;

	post(msg: any, eventType: string): void;

	register(observer: any, sticky: boolean, eventType: string, subscriber: SubscriberCallback): void;

	unregister(observer: any): void;

	destroy(): void;
}

declare module 'compatible-tools' {
	function getObjectHelper(): ObjectHelper;

	function getDateHelper(): DateHelper;

	function getUrlHelper(): UrlHelper;

	function getCalendarHelper(): CalendarHelper;

	function getCountDownTimer(millisInFuture: number, countDownInterval: number): CountDownTimer;

	function getArrayHelper(): ArrayHelper;

	function getStringHelper(): StringHelper;

	function getEventBus(): EventBus;

	function getWatcherHelper(): WatcherHelper;

	const dateConstants: DateConstants;

	class LogUtils {
		constructor(debug: boolean);

		setEnableDebug(debug: boolean): void;

		debug(msg: any): void;

		debugWithTag(tag: string, msg: any): void;

		info(msg: any): void;

		infoWithTag(tag: string, msg: any): void;

		log(msg: any): void;

		logWithTag(tag: string, msg: any): void

		error(msg: any): void

		errorWithTag(tag: string, msg: any): void

		warn(msg: any): void

		warnWithTag(tag: string, msg: any): void
	}
}
