const dateConstants = require("./DateConstants")

function formatNumber(num) {
	return num > 9 ? num : "0" + num;
}

function getDateInstance(date) {
	if (date instanceof Date) {
		return date;
	}
	if (typeof date === 'string' && date.indexOf('-') > 0) {//兼容ios new Date('2020-10-10') 返回Invalid Date
		const reg = new RegExp("-", "g");
		date = date.replace(reg, '/')
	}
	return new Date(date);
}

function isToday(date) {
	let currentDate = new Date();
	let orderDate = getDateInstance(date)
	return currentDate.getFullYear() === orderDate.getFullYear() && orderDate.getMonth() === currentDate.getMonth() && orderDate.getDate() === currentDate.getDate();
}

//格式化日期 YYYY-MM-DD
function getFormatDate(date) {
	const dateInstance = getDateInstance(date);
	return dateInstance.getFullYear() + "-" + formatNumber(dateInstance.getMonth() + 1) + "-" + formatNumber(dateInstance.getDate());
}

//获取当前日期
function getCurrentFormatDate() {
	return getFormatDate(new Date())
}

//获取当前日期所在月份的天数
function getDaysInMonth(date) {
	const dateInstance = getDateInstance(date);
	return getMonthDays(dateInstance.getFullYear(), dateInstance.getMonth() + 1);
}

function getWeekName(date, isEnglish = false) {
	const dateInstance = getDateInstance(date);
	const weekDay = dateInstance.getDay()
	if (weekDay === 0) {
		return isEnglish ? 'Sunday' : '周日';
	} else if (weekDay === 1) {
		return isEnglish ? 'Monday' : "周一";
	} else if (weekDay === 2) {
		return isEnglish ? 'Tuesday' : '周二';
	} else if (weekDay === 3) {
		return isEnglish ? 'Wednesday' : '周三';
	} else if (weekDay === 4) {
		return isEnglish ? 'Thursday' : '周四';
	} else if (weekDay === 5) {
		return isEnglish ? 'Friday' : '周五';
	} else if (weekDay === 6) {
		return isEnglish ? 'Saturday' : '周六';
	}
	return '';
}

function isLeapYear(year) {
	return year % 400 === 0 || (year % 4 === 0 && year % 100 !== 0);
}

/**
 * 获取指定月份天数
 * @param year
 * @param month
 * @returns {number}
 */
function getMonthDays(year, month) {
	if (month === 2 || month === "2") {
		return isLeapYear(year) ? 29 : 28;
	}
	const bigMonths = [1, 3, 5, 7, 8, 10, 12]
	if (bigMonths.includes(month)) {
		return 31;
	}
	return 30;
}

function getNextMonthInfo(year, month) {
	const newMonth = month == 12 ? 1 : month + 1;
	const newYear = month == 12 ? year + 1 : year;
	return {year: newYear, month: newMonth}
}

function getLastMonthInfo(year, month) {
	//获取上个月实际月份
	const newMonth = month == 1 ? 12 : month - 1;
	//获取实际年份
	const newYear = month == 1 ? year - 1 : year;
	return {year: newYear, month: newMonth}
}

function diffTime(startTime, endTime, unitTime) {
	if (typeof startTime === 'number' && typeof endTime === 'number') {
		const diffTimeNum = endTime - startTime
		return Math.floor((diffTimeNum / unitTime) * 100) / 100
	}
	return undefined;
}

function diffYears(startTime, endTime) {
	let number = diffMonths(startTime, endTime);
	return Math.floor(number / 12 * 100) / 100;
}

function diffMonths(startTime, endTime) {
	let miniTime = Math.min(startTime, endTime);
	let maxTime = Math.max(startTime, endTime);
	const startDate = getDateInstance(miniTime);
	const endDate = getDateInstance(maxTime);
	let diffYear = endDate.getFullYear() - startDate.getFullYear();
	let startMonth = startDate.getMonth() + 1;
	let endMonth = endDate.getMonth() + 1;
	if (endMonth < startMonth) {
		diffYear = diffYear - 1;
		endMonth = endMonth + 12;
	}
	return Number.parseInt((diffYear * 12 + endMonth - startMonth).toFixed(0))
}

function diffDays(startTime, endTime) {
	return diffTime(startTime, endTime, dateConstants.dayMillis)
}

function diffHours(startTime, endTime) {
	return diffTime(startTime, endTime, dateConstants.hourMillis)
}

function diffMinutes(startTime, endTime) {
	return diffTime(startTime, endTime, dateConstants.minuteMillis)
}

function diffSeconds(startTime, endTime) {
	return diffTime(startTime, endTime, dateConstants.secondMillis)
}

///yyyy-MM-dd HH:mm:ss
const formatDateList = [
	{
		symbolList: ["yyyy", "YYYY"],
		getValue: function (date) {
			return formatNumber(date.getFullYear());
		}
	},
	{
		symbolList: ["MM"],
		getValue: function (date) {
			return formatNumber(date.getMonth() + 1);
		}
	},
	{
		symbolList: ["DD", "dd"],
		getValue: function (date) {
			return formatNumber(date.getDate());
		}
	},
	{
		symbolList: ["HH", "hh"],
		getValue: function (date) {
			return formatNumber(date.getHours());
		}
	},
	{
		symbolList: ["mm"],
		getValue: function (date) {
			return formatNumber(date.getMinutes());
		}
	},
	{
		symbolList: ["SS", "ss"],
		getValue: function (date) {
			return formatNumber(date.getSeconds());
		}
	},
]

function formatDateString(date, formatStyle) {
	let format = String(formatStyle);
	if (formatStyle) {
		let dateInstance = getDateInstance(date);
		formatDateList.forEach(formatItem => {
			for (let i = 0, len = formatItem.symbolList.length; i < len; i++) {
				if (format.indexOf(formatItem.symbolList[i]) >= 0) {
					format = format.replace(formatItem.symbolList[i], formatItem.getValue(dateInstance))
					break;
				}
			}
		})
	}
	return format;
}

module.exports = {
	getDateInstance: getDateInstance,
	isToday: isToday,
	formatNumber: formatNumber,
	formatDateString: formatDateString,
	getCurrentFormatDate: getCurrentFormatDate,
	getDaysInMonth: getDaysInMonth,
	getMonthDays: getMonthDays,
	getWeekName: getWeekName,
	isLeapYear: isLeapYear,
	getLastMonthInfo: getLastMonthInfo,
	getNextMonthInfo: getNextMonthInfo,
	diffYears: diffYears,
	diffMonths: diffMonths,
	diffDays: diffDays,
	diffHours: diffHours,
	diffMinutes: diffMinutes,
	diffSeconds: diffSeconds,
};

